@@ -10,13 +10,13 @@ from BDMLtools.selector import LgbmSeqSelector,LgbmPISelector,LgbmShapRFECVSelec
 #from BDMLtools.selector import RFECVSelector
 from BDMLtools.selector import lassoSelector
 from BDMLtools.plotter import BaseWoePlotter
-from BDMLtools.plotter import perfEval
+from BDMLtools.plotter import perfEval,perfEval2
 from BDMLtools.encoder import woeTransformer
 from BDMLtools.tuner import gridTuner,hgridTuner
 from BDMLtools.tuner import BayesianCVTuner,shapCheck
 
 
-__version__ = '0.2.3'
+__version__ = '0.2.5'
 
 __all__ = (    
     dtStandardization,
@@ -52,6 +52,7 @@ __all__ = (
     hgridTuner,
     BayesianCVTuner,
     perfEval,
+    perfEval2,
     shapCheck
 )
 
