@@ -105,6 +105,15 @@ class Base:
         if not dtype in ('float32','float64'):
             
             raise ValueError("dtype in ('float32','float64')")
+            
+            
+    def _check_yname(self,y):
+        
+        if y.name is None:
+            
+            raise ValueError("Series y must have a name!")
+            
+        
 
 
 
@@ -163,7 +172,7 @@ class BaseEval:
         
         if pd.isnull(y_pred).any():
             
-            warn("y_pred contains NAN and will be droped.")   
+            raise ValueError("y_pred contains NAN")   
             
         if pd.isnull(y_true).any():
             
@@ -177,7 +186,7 @@ class BaseEval:
             
             if pd.isnull(group).any():
             
-                warn("group contains NAN and will be droped.")  
+                raise ValueError("group contains NAN")  
 
         if not np.isin(y_true.unique(),[0,1]).any():
         
