@@ -304,7 +304,7 @@ class preSelector(Base,Specials,TransformerMixin):
         
         keep_col=self.features_info['0.orgin']
         
-        print('0.start__________________________________complete')
+        print('Start'.center(100,'-'))
         
         step=0
       
@@ -318,7 +318,7 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyNA']=keep_col
           
-            print(str(step)+'.filterbyNA_____________________________complete')
+            print((str(step)+'.filterbyNA').rjust(35)+'Done.'.rjust(35,'_'))
             
         #fliter by unique_pct
         if self.unique_pct is not None: 
@@ -329,7 +329,7 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyUnique']=keep_col
             
-            print(str(step)+'.filterbyUniquepct______________________complete')    
+            print((str(step)+'.filterbyUniquepct').rjust(42)+'Done.'.rjust(28,'_'))
                       
         #fliter by variance
         if self.variance is not None: 
@@ -340,7 +340,7 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyVariance']=keep_col
             
-            print(str(step)+'.filterbyVariance_______________________complete')
+            print((str(step)+'.filterbyVariance').rjust(41)+'Done.'.rjust(29,'_'))
             
        
         #fliter by chi and f-value
@@ -352,7 +352,7 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyChi2Oneway']=keep_col
             
-            print(str(step)+'.filterbyChi2Oneway_____________________complete')     
+            print((str(step)+'.filterbyChi2Oneway').rjust(43)+'Done.'.rjust(27,'_'))
             
             
         #fliter by pi
@@ -365,7 +365,7 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyPermutationImp']=keep_col
 
-            print(str(step)+'.filterbyPermutationImp_________________complete')  
+            print((str(step)+'.filterbyPermutationImp').rjust(47)+'Done.'.rjust(23,'_')) 
         
         #fliter by LOFO_importannce
         if self.lofoi_limit is not None:
@@ -376,7 +376,7 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyLOFOImp']=keep_col
 
-            print(str(step)+'.filterbyLOFOImp________________________complete')              
+            print((str(step)+'.filterbyLOFOImp').rjust(40)+'Done.'.rjust(30,'_'))              
              
         
         #fliter by lgbm-tree-imp  
@@ -388,7 +388,7 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyTrees']=keep_col
             
-            print(str(step)+'.filterbyTrees__________________________complete')
+            print((str(step)+'.filterbyTrees').rjust(38)+'Done.'.rjust(32,'_'))
             
         
         #fliter by iv 
@@ -400,15 +400,15 @@ class preSelector(Base,Specials,TransformerMixin):
             
             self.features_info[str(step)+'.filterbyIV']=keep_col
             
-            print(str(step)+'.filterbyIV_____________________________complete')
+            print((str(step)+'.filterbyIV').rjust(35)+'Done.'.rjust(35,'_'))
    
         
-        print('_____________________________________________Done')  
+        print('Done'.center(100,'-'))
         
         #summary
         for key in self.features_info:
             
-            print('步骤{},保留的特征数:{}'.format(key,len(self.features_info[key])))
+            print('Step:{},features remain:{}'.format(key,len(self.features_info[key])))
         
         #report
         if self.out_path: 
@@ -425,6 +425,24 @@ class preSelector(Base,Specials,TransformerMixin):
     
     
     def _filterByNA(self,X,na_pct,ws=None):
+        """
+        使用缺失值比例筛选数值
+        + 缺失值指代为np.nan,即函数只会识别float类型的np.nan为缺失值
+        + 对于分类特征
+            + string型中np.nan为字符'nan'，因此其不会被当作np.nan处理
+            + object型中np.nan可以为float,因此其可以被当作np.nan处理
+            + category型中np.nan可以为float,因此其可以被当作np.nan处理
+
+        Parameters:
+        ----------  
+        X:pandas.DataFrame,X特征
+        unique_pct:float,唯一值比例阈值，大于等于阈值的列将被筛除
+        ws:Series,样本权重
+            
+        Return:
+        ---------- 
+        list,筛选后的列名
+        """
         
         if ws is None:
             
@@ -444,6 +462,19 @@ class preSelector(Base,Specials,TransformerMixin):
     
     
     def _filterByUnique(self,X,unique_pct,ws=None):
+        """
+        使用唯一值比例筛选数值、分类特征
+        
+        Parameters:
+        ----------  
+        X:pandas.DataFrame,X特征
+        unique_pct:float,唯一值比例阈值，大于等于阈值的列将被筛除
+        ws:Series,样本权重
+            
+        Return:
+        ---------- 
+        list,筛选后的列名
+        """
   
         X=X.select_dtypes(include=['object','number'])
         X_oth=X.select_dtypes(exclude=['object','number'])
@@ -469,6 +500,20 @@ class preSelector(Base,Specials,TransformerMixin):
 
     
     def _fliterByVariance(self,X,variance,ws=None):
+        """
+        使用方差筛选数值特征，
+        + 本函数只筛选数值特征，分类特征不进行处理直接输出
+        
+        Parameters:
+        ----------  
+        X:pandas.DataFrame,X特征
+        variance:float,方差阈值，小于等于阈值的列将被筛除
+        ws:Series,样本权重
+            
+        Return:
+        ---------- 
+        list,筛选后的列名
+        """
 
         X_numeric=X.select_dtypes('number')
         X_oth=X.select_dtypes(exclude='number')
@@ -520,7 +565,7 @@ class preSelector(Base,Specials,TransformerMixin):
             sklearn.feature_selection.chi2
             sklearn.feature_selection.f_classif
         
-        从统计角度上，此两种方法都对数据有分布要求，且f_classif默认样本的方差出自同一总体,若偏离要求太多则p-value意义不大。
+        从统计角度上，此两种方法都要求数据独立同分布，且f_classif默认样本的方差出自同一总体,若偏离要求太多则p-value意义不大。
         因此只使用这两种方法进行特征的初步筛选，目的在于筛选掉非常不重要的特征
         
         筛选依据为这些检验的显著性alpha值或其修正值,并非统计量的值(卡方值或F值),参考:
@@ -604,6 +649,22 @@ class preSelector(Base,Specials,TransformerMixin):
     
 
     def _filterByTrees(self,X,y,tree_size,tree_imps,sample_weight=None):
+        
+        """
+        使用LightGbm的Split重要性筛选特征
+        
+        Parameters:
+        ----------  
+        X:pandas.DataFrame,X特征
+        y:pandas.Seires,目标特征
+        tree_size,int,数个数
+        tree_imps,int,特征重要性阈值，小于等于阈值的列将被剔除
+        sample_weight:Series,样本权重
+            
+        Return:
+        ---------- 
+        list,筛选后的列名
+        """
 
         X_numeric=X.select_dtypes(include='number')
         X_category=X.select_dtypes(include='object')
@@ -653,6 +714,28 @@ class preSelector(Base,Specials,TransformerMixin):
         
     def _filterbyLofoimp(self,X,y,lofo_imp=0,sample_weight=None):
         
+        """
+        使用Leave one out重要性筛选特征:
+            + 特征x的重要性=全量特征模型表在交叉验证集表现的均值-移除特征x后的模型在交叉验证集表现的均值
+            + 基模型为lightgbm
+            + 模型表现衡量标准为roc_auc
+            
+        该重要性计算量较大请慎用
+        
+        LOFOImportance:https://github.com/aerdem4/lofo-importance
+        
+        Parameters:
+        ----------  
+        X:pandas.DataFrame,X特征
+        y:pandas.Seires,目标特征
+        lofo_imp,int,特征重要性阈值，小于等于阈值的列将被剔除
+        sample_weight:Series,样本权重
+            
+        Return:
+        ---------- 
+        list,筛选后的列名
+        """
+        
         X=X.apply(lambda x:x.astype('category') if x.dtypes=='object' else x)  
         
         dt=Dataset(X.join(y),y.name,features=X.columns.tolist())
@@ -663,11 +746,26 @@ class preSelector(Base,Specials,TransformerMixin):
         return fimp[fimp['importance_mean']>lofo_imp]['feature'].tolist()
         
     
-    def _filterbyIV(self,X,y,iv_limit,sample_weight=None):        
+    def _filterbyIV(self,X,y,iv_limit,sample_weight=None): 
+        """
+        使用分箱IV重筛选特征:
+            + 等频30箱
+        
+        Parameters:
+        ----------  
+        X:pandas.DataFrame,X特征
+        y:pandas.Seires,目标特征
+        iv_limit,int,IV重要性阈值，小于等于阈值的列将被剔除
+        sample_weight:Series,样本权重
+            
+        Return:
+        ---------- 
+        list,筛选后的列名
+        """
         
         _,vtabs=binFreq(X,y,ws=sample_weight,bin_num_limit=30)
         
-        iv_t=pd.Series({key:vtabs[key]['total_iv'].max() for key in vtabs},name='iv_infos')
+        iv_t=pd.Series({key:vtabs[key]['total_iv'].max() for key in vtabs},name='iv_infos',dtype='float')
 
         return iv_t[iv_t>iv_limit].index.tolist()
     
